"""
    Browser type detection
    ======= ==== =========

Detect the users browser type and version (Based on an internal list)
from the value of the request variable HTTP_USER_AGENT.

If you add a browser to this list, please send me your additions so
that I can include them in the next release. You will also get a credit
in this DocString!!!

Browser               Contributed by

"""
__version__='4.0'

import string

_browserTag   = [
                {'Text': 'Opera/2', 'Exclude': '', 'Name': 'Opera', 'Version': 2.0},
                {'Text': 'Opera/3', 'Exclude': '', 'Name': 'Opera', 'Version': 3.0},
                {'Text': 'Opera/4', 'Exclude': '', 'Name': 'Opera', 'Version': 4.0},
                {'Text': 'MSIE 3', 'Exclude': '', 'Name': 'MSIE', 'Version': 3.0},
                {'Text': 'MSIE 4', 'Exclude': '', 'Name': 'MSIE', 'Version': 4.0},
                {'Text': 'MSIE 5', 'Exclude': '', 'Name': 'MSIE', 'Version': 5.0},
                {'Text': 'MSIE 5.5', 'Exclude': '', 'Name': 'MSIE', 'Version': 5.5},
                {'Text': 'Mozilla/2', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 2.0},
                {'Text': 'Mozilla/3', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 3.0},
                {'Text': 'Mozilla/4', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 4.0},
                {'Text': 'Mozilla/5', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 5.0},
                {'Text': 'Mozilla/6', 'Exclude': 'MSIE', 'Name': 'Netscape', 'Version': 6.0},
                {'Text': 'Lynx/2.7', 'Exclude': '', 'Name': 'Lynx', 'Version': 2.7},
                {'Text': 'Lynx/2.8', 'Exclude': '', 'Name': 'Lynx', 'Version': 2.8},
                ]

def ListBrowsers():
    """
        Return a list of the known browsers plus the catch all 'Other'
    """
    List = []
    for ThisBrowser in _browserTag:
        ThisName="%s" % ThisBrowser["Name"]
        if not(ThisName in List):
            List.append(ThisName)
    List.append("Other")
    return List

class BrowserType:
    """
        Browser type detection mix-in class
    """

    def _splitBrowser(self):
        """
            Split the REQUEST HEADER (HTTP_USER_AGENT)
            to crunch out the name and version
        """
        ua="%s" % self.REQUEST["HTTP_USER_AGENT"]
        Name=""
        Version=0
        for ThisBrowser in _browserTag:
            if (string.find(ua, ThisBrowser["Text"])!=-1):
                if ((string.find(ua, ThisBrowser["Exclude"])==-1) or
                    (not ThisBrowser["Exclude"])):
                    Name="%s" % ThisBrowser["Name"]
                    Version=ThisBrowser["Version"] * 1.0
        if (Name==""):
            Name="Other"
            Version=1.0
        return (Name, Version)

    def BrowserName(self):
        """
            Return the NAME of the browser type
        """
        (Name, Version) = self._splitBrowser()
        return Name

    def BrowserVersion(self):
        """
            Return the NAME of the browser type
        """
        (Name, Version) = self._splitBrowser()
        return Version

    def isBrowser(self, BrowserName):
        """
            Returns 'Browser Version' if this is the
            named browser, 0 otherwise (Float)
        """
        (Name, Version) = self._splitBrowser()
        if (Name == BrowserName):
            return Version
        return 0

    def BrowserList(self):
	"""
	    Return a list of the known browsers plus the catch all 'Other'
	"""
	return ListBrowsers()

    def BrowserCount(self):
        """
            Return the number of items in the BrowserList
        """
        return len(ListBrowsers())
