"""
    ZSS Rule
    === ====

A Zope object, representing one or more CSS selectors,
and containing zero or more ZSSRule and/or ZSSAttribute
objects. The selectors represented are combined with the
selectors in the parent (And higher) nodes to produce a
(potentially very long) list of contextual selectors.
"""
__version__ = '4.0'

from Globals import HTMLFile, MessageDialog
from OFS import SimpleItem 
from OFS import ObjectManager
from OFS import PropertyManager
from DocumentTemplate import DT_HTML

import re
import string

import internals
import BrowserType
import ZSSAttribute

manage_addZSSRuleForm=HTMLFile('www/manage_addZSSRule', globals())

def manage_addZSSRule(self, id, title='', selector=[], Browsers=[], REQUEST=None):
    """
        Add a ZSSRule object
    """
    obj=ZSSRule(id, title, selector, Browsers, REQUEST)
    self._setObject(obj.id, obj)
    if REQUEST is not None:
        if REQUEST.has_key('submit'):
            if REQUEST['submit']==' Add and Edit ':
                REQUEST.RESPONSE.redirect("%s/%s/manage_main" % (REQUEST['URL1'], obj.id))
                return ''
        REQUEST.RESPONSE.redirect("%s/manage_main" % REQUEST['URL1'])
    return ''

manageAddForm = manage_addZSSRuleForm
manageAdd = manage_addZSSRule

class ZSSRule(	ObjectManager.ObjectManager,
                PropertyManager.PropertyManager,
                SimpleItem.SimpleItem,
                BrowserType.BrowserType):
    """
        Representation of Cascading Style Sheet Rules.
        Parent of ZSSRule, ZSSSelector and ZSSAttribute objects.
    """
    manage_addZSSRuleForm=manageAddForm
    manage_addZSSRule=manageAdd
    manage_addZSSAttrForm=ZSSAttribute.manage_addZSSAttrForm
    manage_addZSSAttr=ZSSAttribute.manage_addZSSAttr
    manage_ChangeBrowserForm=HTMLFile('www/manage_ChangeBrowser', globals())

    meta_type      = 'ZSS Rule'
    id             = 'ZSSRule'
    icon           = 'misc_/ZStyleSheet/ZSSRule'
    all_meta_types = meta_types = (
        {'name': 'ZSS Rule', 'action': 'manage_addZSSRuleForm'},
        {'name': 'ZSS Attribute', 'action': 'manage_addZSSAttrForm'},
    )
    manage_options = (
        ObjectManager.ObjectManager.manage_options
        +PropertyManager.PropertyManager.manage_options
        +(
            {'label': 'View', 'action': 'preview',
             'help': ('ZStyleSheet', 'Preview.stx')},
            {'label': 'Dump', 'action': 'dump',
             'help': ('ZStyleSheet', 'Dump.stx')},
            {'label': 'Change Browser', 'action': 'manage_ChangeBrowserForm',
             'help': ('ZStyleSheet', 'ChangeBrowser.stx')},
        )
        +SimpleItem.SimpleItem.manage_options
    )
    __ac_permissions__= (
        PropertyManager.PropertyManager.__ac_permissions__
        +SimpleItem.SimpleItem.__ac_permissions__
    )
    _properties=(
        {'id' : 'title', 'type': 'string', 'mode': 'w'},
        {'id' : 'selector', 'type': 'lines', 'mode': 'w'},
        {'id' : 'Browsers', 'type' : 'multiple selection', 'mode': 'w', 'select_variable': 'BrowserList'},
    )

    def __init__(self, id, title='', selector=[], Browsers=[], REQUEST=None):
        """
            Initialise a ZSSRule
        """
        if id:
            self.id=id
        else:
            self.id=internals.translateSelector(selector)
        self.title=title
        self.selector=selector
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers

    def manage_ChangeBrowser(self, Browsers=[], REQUEST=[]):
        """
            Walk the tree, changing the Browsers entry at each node
        """
        if (not(Browsers)):
            Browsers=self.BrowserList()
        self.Browsers=Browsers
        objs=self.objectValues('ZSS Rule')
        for i in objs:
            i.manage_ChangeBrowser(Browsers)
        objs=self.objectValues('ZSS Attribute')
        for i in objs:
            i.Browsers=Browsers
        if REQUEST:
            return MessageDialog(
                title  ='Success!',
                message='The browser settings have been changed',
                action ='manage_main')

    def calccontext(self, parentcontext=[], REQUEST=None):
        """
            Calculate the context relative to the passed parent context(s)
        """
        context=[]
        if parentcontext:
            for i in parentcontext:
                for s in self.selector:
                    context.append(i+" "+s)
        else:
            for s in self.selector:
                context.append(s)
        return context

    def _dump(self, ignoreBrowsers=1, call='_dump'):
        """ Add this rule to a dump listing """
        return "/* Browsers=%s */\n%s" % (self.Browsers, self._value(ignoreBrowsers=ignoreBrowsers, call=call),)

    def _children(self, TYPE='', HEAD='', TAIL='', ignoreBrowsers=0, call='_value'):
        """ This does the children of this item """
        strret = ""
        objs = self.objectValues(TYPE)
        objs.sort(lambda x, y: cmp(x.id, y.id))
        for i in objs:
            Icall = getattr(i, call)
            strret = strret + "%s%s%s" % (HEAD, Icall(ignoreBrowsers=ignoreBrowsers, call=call), TAIL)
        return strret

    def _value(self, parentcontext=[], ignoreBrowsers=0, call='_value'):
        """ Display the stylesheet from here down """
        strret=""
        if self.BrowserName() in self.Browsers or ignoreBrowsers:
            context=self.calccontext(parentcontext)
            attribs = self._children(TYPE='ZSS Attribute', HEAD='\t', TAIL=(call=='_value') and '\n\n' or '\n', ignoreBrowsers=ignoreBrowsers, call=call)
            if attribs:
                strret = string.join(context, ", ") + " {\n%s}\n" % attribs
            strret = strret + self._children(TYPE='ZSS Rule', HEAD='', TAIL='\n', ignoreBrowsers=ignoreBrowsers, call=call)
        else:
            strret = "/* %s - Excluded for this browser */" % self.id
        return strret

    def dump(self):
        """ Dump this rule, showing the browser restrictions in comments. """
        dtml = '<dtml-var manage_page_header><dtml-var manage_tabs>'
        dtml = dtml + "<h1 align=center>Zope Style Sheet Rule Dump</h1>"
        dtml = dtml + "<h2 align=center>%s</h2>" % self.title_and_id()
        dtml = dtml + "<blockquote><pre>%s</pre></blockquote>" % self._dump()
        dtml = dtml + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        dtml = dtml + "<dtml-var manage_page_footer>"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZSSRule - dump - %s' % self.getId())
        return obj(self.aq_inner, self.REQUEST)

    def preview(self):
        """ Render a human viewable version of this ZSSRule """
        dtml = "<dtml-var manage_page_header><dtml-var manage_tabs>\n"
        dtml = dtml + "<h1 align=center>Zope Style Sheet Rule Preview</h1>"
        dtml = dtml + "<h2 align=center>%s</h2>" % self.title_and_id()
        dtml = dtml + "<blockquote><pre>%s</pre></blockquote>" % self._value()
        dtml = dtml + '<h5 align=center>Zope Style Sheet: <a href="http://www.zope.org/Members/haqa/ZStyleSheet">http://www.zope.org/Members/haqa/ZStyleSheet</a></h5>'
        dtml = dtml + "<dtml-var manage_page_footer>"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZSSRule - preview - %s' % self.title_and_id())
        return obj(self.aq_inner, self.REQUEST)

    def tag(self):
        """ Render the attributes of this rule as a 'style=' HTML attribute """
        if self.BrowserName() in self.Browsers:
            obj = DT_HTML.HTML('style="%s"' % self._children(TYPE='ZSS Attribute'), self.REQUEST, 'ZSSRule - tag - %s' % self.title_and_id())
            return obj(self.aq_inner, self.REQUEST)
        return ""

    __str__ = __call__ = tag

    def style(self):
        """ Render the rule as a <style> </style> section """
        if self.BrowserName() in self.Browsers:
            obj = DT_HTML.HTML('<style type="text/css"><!--\n%s\n--></style>' % self._value(), self.REQUEST, 'ZSSRule - style - %s' % self.title_and_id())
            return obj(self.aq_inner, self.REQUEST)
        return ""

    def index_html(self):
        """ Render the rule as text/css as though it were a root ZStyleSheet """
        dtml = """<dtml-call "RESPONSE.setHeader('Content-Type', 'text/css')">"""
        dtml = dtml + "/* Cascading Style Sheet Rule created using ZStyleSheet */\n"
        dtml = dtml + self._children(TYPE='ZSS Rule', HEAD='', TAIL='\n')
        dtml = dtml + "/* Written by Haqa http://www.zope.org/Members/haqa */\n"
        obj = DT_HTML.HTML(dtml, self.REQUEST, 'ZSSRule - index_html - %s' % self.title_and_id()), 
        return obj(self.aq_inner, self.REQUEST)
